/* KallistiGL for KOS 1.1.7

   glmisc.c
   (c)2001 Dan Potter
*/

#include <GL/gl.h>
#include <GL/glu.h>
#include "glinternal.h"
#include "glmacros.h"

#include <assert.h>
#include <stdio.h>

CVSID("$Id: glmisc.c,v 1.11 2002/04/03 03:37:32 axlen Exp $");

/* Miscellaneous APIs */

/* Set the background clear color */
void glClearColor(GLclampf red,
		GLclampf green,
		GLclampf blue,
		GLclampf alpha) {
	assert_msg(!gl_prim_type, "Not allowed within glBegin/glEnd pair.");
	gl_clear_color[0] = CLAMP01(red);
	gl_clear_color[1] = CLAMP01(green);
	gl_clear_color[2] = CLAMP01(blue);
	gl_clear_color[3] = CLAMP01(alpha);
	pvr_set_bg_color(gl_clear_color[0]*gl_clear_color[3],
			gl_clear_color[1]*gl_clear_color[3],
			gl_clear_color[2]*gl_clear_color[3]);
}

/* NOP: there's nothing to clear on the PVR2 */
void glClear(GLbitfield mask) {
	assert_msg(!gl_prim_type, "Not allowed within glBegin/glEnd pair.");
}

/* Set front face */
void glFrontFace(GLenum mode) {
	assert_msg(!gl_prim_type, "Not allowed within glBegin/glEnd pair.");
	switch(mode) {
	case GL_CW:
	case GL_CCW:
		gl_front_face = mode;
		gl_pbuf_submitted = GL_FALSE;
		break;
	}
}

/* Set face culling mode */
void glCullFace(GLenum mode) {
	assert_msg(!gl_prim_type, "Not allowed within glBegin/glEnd pair.");
	switch(mode) {
	case GL_FRONT:
	case GL_BACK:
		gl_cull_mode = mode;
		gl_pbuf_submitted = GL_FALSE;
		break;
	}
}

/* Enable / disable capabilities */
void glEnable(GLenum cap) {
	assert_msg(!gl_prim_type, "Not allowed within glBegin/glEnd pair.");
	switch(cap) {
	case GL_TEXTURE_2D:
		gl_pbuf_submitted = GL_FALSE;
		gl_poly_cxt.txr.enable = PVR_TEXTURE_ENABLE;
		break;
	case GL_BLEND:
		if (gl_active_list != GL_LIST_TRANS_POLY) {
			dbglog(DBG_ERROR, "gl: invalid list to enable GL_BLEND inside");
			return;
		}
		break;
	case GL_CULL_FACE:
		gl_pbuf_submitted = GL_FALSE;
	        gl_cull_face = GL_TRUE;
	        break;
	case GL_FOG:
		gl_pbuf_submitted = GL_FALSE;
		gl_poly_cxt.gen.fog_type = PVR_FOG_TABLE;
		break;
	case GL_SCISSOR_TEST:
		gl_pbuf_submitted = GL_FALSE;
		gl_poly_cxt.gen.clip_mode = PVR_USERCLIP_INSIDE;
		if (USERCLIP_OUTSIDE_ENABLED)
			gl_capabilities &= ~((GLbitfield)GL_KOS_USERCLIP_OUTSIDE);
		break;
	case GL_KOS_USERCLIP_OUTSIDE:
		gl_pbuf_submitted = GL_FALSE;
		gl_poly_cxt.gen.clip_mode = PVR_USERCLIP_OUTSIDE;
		if (SCISSOR_ENABLED)
			gl_capabilities &= ~((GLbitfield)GL_SCISSOR_TEST);
		break;
	}

	gl_capabilities |= (GLbitfield)cap;
}

void glDisable(GLenum cap) {
	assert_msg(!gl_prim_type, "Not allowed within glBegin/glEnd pair.");
	switch(cap) {
	case GL_TEXTURE_2D:
		gl_pbuf_submitted = GL_FALSE;
		gl_poly_cxt.txr.enable = PVR_TEXTURE_DISABLE;
		gl_cur_texture = &gl_null_texture;
		break;
	case GL_BLEND:
		if (gl_active_list == GL_LIST_TRANS_POLY) {
			dbglog(DBG_ERROR, "gl: invalid list to disable GL_BLEND inside");
			return;
		}
		break;
	case GL_CULL_FACE:
		gl_pbuf_submitted = GL_FALSE;
	        gl_cull_face = GL_FALSE;
	        break;
	case GL_FOG:
		gl_pbuf_submitted = GL_FALSE;
		gl_poly_cxt.gen.fog_type = PVR_FOG_DISABLE;
		break;
	case GL_SCISSOR_TEST:
		gl_pbuf_submitted = GL_FALSE;
		gl_poly_cxt.gen.clip_mode = PVR_USERCLIP_DISABLE;
		if (USERCLIP_OUTSIDE_ENABLED)
			gl_capabilities &= ~((GLbitfield)GL_KOS_USERCLIP_OUTSIDE);
		break;
	case GL_KOS_USERCLIP_OUTSIDE:
		gl_pbuf_submitted = GL_FALSE;
		gl_poly_cxt.gen.clip_mode = PVR_USERCLIP_DISABLE;
		if (SCISSOR_ENABLED)
			gl_capabilities &= ~((GLbitfield)GL_SCISSOR_TEST);
		break;
	}

	gl_capabilities &= ~((GLbitfield)cap);
}

/* We have no rendering pipeline yet, so this is a NOP */
void glFlush() {
	assert_msg(!gl_prim_type, "Not allowed within glBegin/glEnd pair.");
}

/* There aren't any useful hints to set yet, so this is a NOP */
void glHint(GLenum target, GLenum mode) {
	assert_msg(!gl_prim_type, "Not allowed within glBegin/glEnd pair.");
}

void glPointSize(GLfloat size) {
	assert_msg(!gl_prim_type, "Not allowed within glBegin/glEnd pair.");
	gl_point_size = CLAMP(size, 0.0f, 100.0f);
}

/* These strings should go somewhere else */
const GLubyte *glGetString(GLenum name) {
	assert_msg(!gl_prim_type, "Not allowed within glBegin/glEnd pair.");
	switch(name) {
		case GL_VENDOR:
			return "Cryptic Allusions";

		case GL_RENDERER:
			return "KallistiOS";

		case GL_VERSION:
			return "1.1";

		case GL_EXTENSIONS:
			return "GL_ARB_transpose_matrix ";
	}

	return "";
}

