/* KallistiOS 1.1.7

   initall.c
   (c)2001-2002 Dan Potter

   This is a shim expected to be removed from KOS 1.1.8.
*/

/*

 This used to do a whole lot more, but at this point these are just compat
 functions in case someone sets up a romdisk via kos_init_all().

 */

#include <arch/arch.h>
#include <kos/fs_romdisk.h>
#include <kos/thread.h>
#include <dc/pvr.h>
#include <dc/ta.h>
#include <stdio.h>

CVSID("$Id: initall.c,v 1.11 2002/03/23 21:44:02 bardtx Exp $");

int kos_init_all(uint32 enables, void *romdisk) {
	dbglog(DBG_DEBUG, "**NOTE** Your program uses kos_init_all(), which is deprecated.\n");

	/* If they supplied us a romdisk, patch it in now */	
	if (romdisk != NULL) {
		__kos_romdisk = romdisk;
		fs_romdisk_init(__kos_romdisk);
	}

	/* Did we get threading wrong? */
	if ((enables & THD_ENABLE) && !(__kos_init_flags & INIT_THD_PREEMPT)) {
		thd_set_mode(THD_MODE_PREEMPT);
	} else if (!(enables & THD_ENABLE) && (__kos_init_flags & INIT_THD_PREEMPT)) {
		thd_set_mode(THD_MODE_COOP);
	}

	/* Did we get PVR wrong? */
	if ((enables & PVR_ENABLE) && !(__kos_init_flags & TA_ENABLE)) {
		ta_init_defaults();
	}

	return 0;
	
#if 0	/* Old stuff, if you want it manually */
	/* If DC-Load is in use, then we'll want to enable that
	   before anything else happens. */
	fs_dcload_init_console();

	/* Do the banner */
	dbgio_init();
	dbglog(DBG_INFO, "\n--\n");		/* Spacer for multiple sessions */
	dbglog(DBG_INFO, banner);

	/* Initialize all of the basic hardware systems */
	irq_init();		/* IRQ */
	timer_init();		/* Timers */
	hardware_init();	/* DC Hardware */
	syscall_init();		/* System call interface */

	/* Initialize VFS */
	dbglog(DBG_KDEBUG, "Initializing VFS\n");
	fs_init();
	if (romdisk != NULL) {
		dbglog(DBG_KDEBUG, "Initializing RD file system\n");
		fs_romdisk_init(romdisk);
	}
	if (*DCLOADMAGICADDR == DCLOADMAGICVALUE) {
		dbglog(DBG_KDEBUG, "Initializing DCLOAD Console file system\n");
		fs_dcload_init();
	}
	dbglog(DBG_KDEBUG, "Initializing CD file system\n");
	fs_iso9660_init();
	dbglog(DBG_KDEBUG, "Initializing VMU file system\n");
	fs_vmu_init();

	/* Enable threads */
	if (enables & THD_ENABLE) {
		dbglog(DBG_KDEBUG, "Initializing threads\n");
		thd_init();
	}

	/* Enable IRQs */
	if (enables & IRQ_ENABLE) {
		dbglog(DBG_KDEBUG, "Enabling IRQs\n");
		irq_enable();
		maple_wait_scan();
	}

	/* Enable networking */
	if (enables & NET_ENABLE) {
		dbglog(DBG_KDEBUG, "Enabling networking\n");
		net_init();
	}

	/* This has to be done AFTER thread init, since it changes its
	   functionality to match the state of threading */
	/* spu_dma_init(); */

	/* Make sure we're not going to foible here */
	assert_msg(!( (enables & TA_ENABLE) && (enables & PVR_ENABLE) ),
		"PVR and TA are mutually exclusive subsystems; cannot proceed");

	/* Potentially enable TA */
	if (enables & TA_ENABLE) {
		dbglog(DBG_KDEBUG, "Initializing 3D system [ta]\n");
		/* dbglog(DBG_WARNING, "  The 'TA' system is deprecated in favor of 'PVR'\n"); */	/* not yet =) */
		ta_init_defaults();
	}

	/* Potentially enable PVR */
	if (enables & PVR_ENABLE) {
		/* This is about as close as we can get to reasonable defaults */
		pvr_init_params_t pvr_params = {
			{ PVR_BINSIZE_32, PVR_BINSIZE_8, PVR_BINSIZE_16, PVR_BINSIZE_8, PVR_BINSIZE_16 },
			512 * 1024
		};
		                
		dbglog(DBG_KDEBUG, "Initializing 3D system [pvr]\n");
		pvr_init(&pvr_params);
	}

	/* Enable millisecond timer */
	dbglog(DBG_KDEBUG, "Initializing Millisecond Timer\n");
	timer_ms_enable();

	return 0;
#endif
}

void kos_shutdown_all() {
#if 0
	irq_disable();
	irq_enable_exc();
	/* spu_dma_shutdown(); */
	net_shutdown();
	thd_shutdown();
	pvr_shutdown();
	ta_shutdown();
	fs_dcload_shutdown();
	fs_vmu_shutdown();
	fs_iso9660_shutdown();
	fs_romdisk_shutdown();
	fs_shutdown();
	irq_shutdown();
#endif
}

